
//////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2024 Autodesk, Inc.  All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.   
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __EDGESYM_H
#define __EDGESYM_H
#ifndef __SYMBOL_H
#error Include "symbol.h" before including this file
#endif

class AmdtCEdge;
class AmdtCEdgeImp;
class AcmSymbolStd;
class AmdtEdgeStdImp;

///<summary>The Edge Standard class. This class is derived from the AcmSymbolStd class.</summary>
class ACMSYMBB_CRX_DLLSHARE AmdtEdgeStd : public AcmSymbolStd
{
  friend class AcmSymbbInternals;
  friend class AcmStandardImp;
  friend class AmdtEdgeStdImp;
public:
  ACRX_DECLARE_MEMBERS(AmdtEdgeStd);
  ///<summary>the destruct function.</summary>
  virtual ~AmdtEdgeStd();
  Adesk::Boolean majorityAllowed() const;
  void setMajorityAllowed(Adesk::Boolean val);
  Adesk::Boolean fullList() const;
  void setFullList(Adesk::Boolean v);
  virtual Acm::ErrorStatus getDescription(CString& desc,
                                          CBitmap*& pBmpLoaded,
                                          CBitmap*& pBmpReg,
                                          Adesk::Boolean& enablePref) const;
  virtual Acm::ErrorStatus checkVisibility(AmdtCEdge* pData) const;
  virtual Acm::ErrorStatus setSuggested(AmdtCEdge *pSubSym) const;
  
  virtual void setDefaultSubSymbol(AmdtCEdge* pData) const;
  virtual Acm::ErrorStatus initFields(AmdtCEdge *pSubSym) const;
  virtual Acm::ErrorStatus validityFields(AmdtCEdge *pSubSym,
                                          bool setBoundary = false,
                                          int dlgIndex = -1);
  virtual void getSubEnts(const AmdtCEdge* pSymb,
                          AcmCAuxEntArray& subents,
                          bool corrSymbOnRightSide = false) const;

  virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
  virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;
  virtual Acad::ErrorStatus dxfInFields(AcDbDxfFiler* filer);
  virtual Acad::ErrorStatus dxfOutFields(AcDbDxfFiler* filer) const;
  ///<summary>This is record AmdtEdgeStd::Variable.</summary>
  enum Variable {
    kBracketAllowed = 0,
    kFullList    
  };
  virtual Acm::Visibility variableStatus(AmdtEdgeStd::Variable var) const;
  virtual void setVariableStatus(AmdtEdgeStd::Variable var, Acm::Visibility status);
  virtual void getHeights(const AmdtCEdge*, AcGeDoubleArray&) const;
  Adesk::Boolean allAroundAllowed() const;
  void setAllAroundAllowed(Adesk::Boolean val);
  Adesk::Boolean stdNumAllowed() const;
  void setStdNumAllowed(Adesk::Boolean val);

  ///<summary>Get the suggested values list for edge.</summary>
  ///<param name="strList">suggest values string list.</param>
  virtual Acm::ErrorStatus getSuggestedList(CStringList& strList) const;

  ///representation for the revision.
  ///<summary>It returns true if the edge symbol is drawn above the arrow landing.</summary>
  virtual bool isSymAboveLanding() const;

protected:
  AmdtEdgeStd();
  virtual void getMajoritySymbol(const AmdtCEdge* pSubSym,
                                 AcmCAuxEntArray& subents) const;
  virtual void getLeaderSymbol(const AmdtCEdge* pSubSym,
                               AcmCAuxEntArray& subents) const;
  virtual Acad::ErrorStatus subGetClassID(CLSID* pClsid) const;

private:
  AmdtEdgeStdImp* mpImp;
};
DBREF_DECLARE(AmdtEdgeStd)

///<summary>The AmdtCEdge class - derived from the AcmCSubSymbol class.</summary>
class AmdtCEdge : public AcmCSubSymbol
{
public:
  virtual Adesk::Boolean isMajority() const = 0;
  virtual Acm::Direction leaderDirection(Acm::Direction singularity = Acm::kToTop, 
                                         AcGePoint3d peak = AcGePoint3d::kOrigin) const = 0;
  virtual Adesk::Boolean isAllAround() const = 0;
  virtual Acm::ErrorStatus setAllAround(Adesk::Boolean flag) = 0;
  virtual void setNewSymbolRev(Adesk::Boolean newRev = Adesk::kTrue) = 0; 
  virtual Adesk::Boolean newSymbolRev() const = 0;
  virtual Adesk::Boolean isStdNum() const = 0;
  virtual Acm::ErrorStatus setStdNum(Adesk::Boolean flag) = 0;
  ///<summary>It returns true if the undefined size is allowed.</summary>
  virtual bool isUndefinedSize() const = 0;
  ///<summary>This sets whether or not the undefined size of the edge symbol is allowed.</summary>
  virtual Acm::ErrorStatus setUndefinedSize(bool flag) = 0;

  ///<summary>It returns true if the direction of undefined size is set.</summary>
  virtual bool isDefinedDirect() const = 0;
  ///<summary>It sets whether or not the direction of undefined size is set.</summary>
  virtual Acm::ErrorStatus setDefinedDirect(bool flag) = 0;  

  ///<summary>It returns the direction of undefined size.</summary>
  ///<returns>The return value can be any member of enum Acm::EdgeUndefinedDirection.</returns>
  virtual Acm::EdgeUndefinedDirection undefineDirection() const = 0;
  ///<summary>Set the direction of undefined size.</summary>
  ///<param name="direct">Input Only.Can be any member of enum Acm::EdgeUndefinedDirection.</param>
  virtual Acm::ErrorStatus setUndefinedDirection(Acm::EdgeUndefinedDirection direct) = 0;
};

///<summary>Class for Note Symbol.</summary>
class ACMSYMBB_CRX_DLLSHARE AmdtEdge : public AcmSymbol
{
friend class CLeaderPage;
public:
  ACRX_DECLARE_MEMBERS(AmdtEdge);
  ///<summary>The construct function.</summary>
  AmdtEdge();
  ///<summary>The destruct function.</summary>
  virtual ~AmdtEdge();
  AmdtCEdge* subSymbol() const;
  LPCTSTR value(Acm::Variable var) const;
  Acm::ErrorStatus setValue(Acm::Variable var, LPCTSTR value);
  Acm::Visibility  variableStatus(Acm::Variable var) const;
  Acm::ErrorStatus setVariableStatus(Acm::Variable var, Acm::Visibility status);
  virtual Acm::ErrorStatus setSymbolDefaults(AcDbDatabase *pPropertiesDb = NULL);
  virtual AmdtCEdge* newSubEdge() const;
  Adesk::Boolean isMajority() const;
  Acm::ErrorStatus setMajority(const Adesk::Boolean);

  using AcmSymbol::transformBy;
  ///<summary>Transforms all leaders. Default implementation doesn't cover any smart behavior.</summary>
  virtual Acad::ErrorStatus transformBy(const AcGeMatrix3d& xform,
                                        double symbolScale);
  ///<summary>This method mirrors AcmSymbol entity.</summary>
  ///<param name="xMirrorForm">Transformation matrix with MIRROR element</param>
  ///<param name="XAxis">Returns mirrored X-Axis of the entity coordinate system</param>
  ///<param name="YAxis">Returns mirrored Y-Axis of the entity coordinate system</param>
  ///<param name="normal">Returns mirrored Normal of the entity coordinate system</param>
  virtual void mirrorCoordSys(const AcGeMatrix3d& xMirrorForm,
                              AcGeVector3d& XAxis,
                              AcGeVector3d& YAxis,
                              AcGeVector3d& normal) const;

  virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
  virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;
  virtual Acad::ErrorStatus dxfInFields(AcDbDxfFiler* filer);
  virtual Acad::ErrorStatus dxfOutFields(AcDbDxfFiler* filer) const;
  ///<summary>Sends a notification if the tip point of the leader has changed.</summary>
  virtual void leaderTipPointChanged(const AcGeVector3d&, const AcmCLeader*);
  Adesk::Boolean getAdditionalEnts(AcmCAuxEntArray& subents) const;
  ///<summary>Provides the method for removing the nth- leader in the list.</summary>
  ///<param name="index">Specifies the index in the leader array. If the leader doesn't exist, 
  ///the function returns eInvalidInput.</param>
  virtual Acm::ErrorStatus removeLeader(Adesk::UInt16 index);
  Adesk::Boolean isAllAround() const;
  Acm::ErrorStatus setAllAround(Adesk::Boolean flag);
  virtual void setNewSymbolRev(Adesk::Boolean newRev = Adesk::kTrue);
  virtual Adesk::Boolean newSymbolRev() const;
  Adesk::Boolean isStdNum() const;
  Acm::ErrorStatus setStdNum(Adesk::Boolean flag);

  ///<summary>It returns true if the undefined size is allowed.</summary>
  bool isUndefinedSize() const;
  ///<summary>This sets whether or not the undefined size of the edge symbol is allowed.</summary>
  Acm::ErrorStatus setUndefinedSize(bool flag);

  ///<summary>It returns true if the direction of undefined size is set.</summary>
  bool isDefinedDirect() const;
  ///<summary>It sets whether or not the direction of undefined size is set.</summary>
  Acm::ErrorStatus setDefinedDirect(bool flag);

  ///<summary>It returns the direction of undefined size.</summary>
  ///<returns>The return value can be any member of enum Acm::EdgeUndefinedDirection.</returns>
  Acm::EdgeUndefinedDirection undefineDirection() const;
  ///<summary>Set the direction of undefined size.</summary>
  ///<param name="direct">Input Only.Can be any member of enum Acm::EdgeUndefinedDirection.</param>
  Acm::ErrorStatus setUndefinedDirection(Acm::EdgeUndefinedDirection direct);

protected:
  virtual Adesk::Boolean    subWorldDraw(AcGiWorldDraw* pMode);
  ///<summary>Queries all leaders for their grip points.</summary>
  virtual Acad::ErrorStatus subGetGripPoints(AcGePoint3dArray&,AcDbIntArray&,AcDbIntArray&) const;
  ///<summary>Queries all leaders for their grip points.</summary>
  virtual Acad::ErrorStatus  subGetGripPoints(AcDbGripDataPtrArray& grips,
                                          const double curViewUnitSize, 
                                          const int gripSize, 
                                          const AcGeVector3d& curViewDir,
                                          const int bitflags) const;
  ///<summary>Determines the leaders to which indices belong and apply moveGripPointsAt to them. If 
  ///the tip point is being edited, then the leaderTipPointChanged() member function of the symbol 
  ///will be called. If the leader contains a landing and it is changed, then the leaderLanding() member
  ///function of the symbol is called.</summary>
  virtual Acad::ErrorStatus subMoveGripPointsAt(const AcDbIntArray&,const AcGeVector3d&);

  virtual Acad::ErrorStatus subMoveGripPointsAt(const AcDbVoidPtrArray& gripAppData, 
                                            const AcGeVector3d& offset, 
                                            const int bitflags);
  ///<summary>Queries the standard for sub-entities for all leaders and then the server for all sub-entities
  ///for all sub-symbols. All sub-entities are then exploded.</summary>
  virtual Acad::ErrorStatus subExplode(AcDbVoidPtrArray& entitySet) const;
  ///<summary>Sums geometry extents for a leader.</summary>
  virtual Acad::ErrorStatus subGetGeomExtents(AcDbExtents& extents) const;
  virtual Acad::ErrorStatus subTransformBy(const AcGeMatrix3d& xform);
  virtual Acad::ErrorStatus subGetClassID(CLSID* pClsid) const;

private:
  AmdtCEdgeImp *mpSubSymbol;
};
DBREF_DECLARE(AmdtEdge)

#endif // __EDGESYM_H
