//////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2024 Autodesk, Inc.  All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.   
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __STDISOEDGE_H
#define __STDISOEDGE_H

///<summary>This derived class implements standard-specific versions of functions available in the AmdtEdgeStd
///class. Refer to the AmdtEdgeStd class for further information.</summary>
class STDIMP_DLLSHARE AmdtEdgeStdISO : public AmdtEdgeStd
{
public:
  ACRX_DECLARE_MEMBERS(AmdtEdgeStdISO);
  ///<summary>The construct function.</summary>
  AmdtEdgeStdISO();
  ///<summary>The destruct function.</summary>
  virtual ~AmdtEdgeStdISO();
  ///<summary>Gets the information about the standard name and revision.</summary>
  virtual Acm::ErrorStatus getRevision(CString& rev, Adesk::UInt16& year) const;
  ///<summary>Provides the basic interface for the AcmStandardManager and hence determines whether AcmSymbolSTD 
  ///can be used for a particular standard.</summary>
  virtual Adesk::Boolean   isCompatibleTo(const AcmStandard* pStd) const;
  virtual void initToEdit();
  virtual void initToEditOriginal();
  ///<summary>Sets the factory default values.</summary>
  virtual Adesk::Boolean setFactoryDefaults();
  virtual Acm::ErrorStatus checkVisibility(AmdtCEdge* pData) const;
  virtual Acm::ErrorStatus initFields(AmdtCEdge* pData) const;
  virtual Acm::ErrorStatus validityFields(AmdtCEdge *pData,
    bool setBoundary = false,
    int dlgIndex = -1);
  virtual void getSubEnts(const AmdtCEdge* pSubSym,
                          AcmCAuxEntArray& subents,
                          bool corrSymbOnRightSide = false) const;
  virtual Acm::ErrorStatus setSuggested(AmdtCEdge* pData) const;

  ///<summary>Get the suggested values list for edge.</summary>
  ///<param name="strList">suggest values string list.</param>
  ///<returns>Returns Acm::eOK.</returns>
  virtual Acm::ErrorStatus getSuggestedList(CStringList& strList)const;

protected:
  virtual void getHeights(const AmdtCEdge*, AcGeDoubleArray&) const;

  ///<summary>It sets the origin point for the text of edge symbol.</summary>
  ///<param name="pText">input pointer</param>
  ///<param name="subEntsText">the symbol's sub-entities array</param>
  ///<param name="basePoint">origin point for text</param>
  ///<param name="str">content</param>
  ///<param name="dir">Attach point for text</param>
  void setTextOrigin(AcmCAuxText* pText,
                     AcmCAuxEntArray& subEntsText,
                     const AcGePoint3d& basePoint,
                     const CString& str, Acm::Direction dir= Acm::kToBottom) const;
};

///<summary>The AmdtEdgeStdISO13715 class is a derived class of the AmdtEdgeStdISO class. The usage of 
///AmdtEdgeStdISO13715 class is the same as that of AmdtEdgeStdISO class.</summary>
class STDIMP_DLLSHARE AmdtEdgeStdISO13715 : public AmdtEdgeStdISO
{
public:
  ACRX_DECLARE_MEMBERS(AmdtEdgeStdISO13715);
  ///<summary>The construct function.</summary>
  AmdtEdgeStdISO13715();
  ///<summary>The destruct function.</summary>
  virtual ~AmdtEdgeStdISO13715();
  ///<summary>Gets the information about the standard name and revision.</summary>
  virtual Acm::ErrorStatus getRevision(CString& rev, Adesk::UInt16& year) const;
  ///<summary>Sets the factory default values.</summary>
  virtual Adesk::Boolean setFactoryDefaults();  
  virtual void getSubEnts(const AmdtCEdge* pSymb, 
                          AcmCAuxEntArray& subents,
                          bool corrSymbOnRightSide = false) const;

  virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);

  ///<summary>Get the symbol's standard number string.</summary>
  ///<param name="rev">standard number.</param>
  virtual void getStandardNum(CString& rev) const;

  ///<summary>It is used to draw standard number for a edge symbol. Add the standard number sub-entities to
  ///edge's sub-entities.</summary>
  ///<param name="pSymb">The edge symbol</param>
  ///<param name="subents">Symbol's sub-entities</param>
  ///<param name="cnvMat">Symbol's transform matrix</param>
  ///<param name="corrSymbOnRightSide">Symbol is on right side of its leader.</param>
  void drawStandardNum(const AmdtCEdge* pSymb, 
                       AcmCAuxEntArray& subents,
                       const AcGeMatrix3d& cnvMat,
                       bool corrSymbOnRightSide) const; 
};

#endif // __STDISOEDGE_H