//////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2024 Autodesk, Inc.  All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.   
//
//////////////////////////////////////////////////////////////////////////////
#ifndef __DATUMTGT_H
#define __DATUMTGT_H

#ifndef __ACMDEF_H
  #error include 'acmdef.h' before including this file
#endif
#ifndef __CAUXENT_H
  #error include 'cauxent.h' before including this file
#endif

#include "AdAChar.h"

class AcmCTarget;
class AcmCTargetBalloon;
class AcmCTargetService;
class AcmCDatumTargetImp;
class AcmDatumStdImp;
///<summary>This class is responsible for datum target sub-symbols. It contains several services that 
///allow it to handle all types of target symbols.</summary>
class ACMSYMBB_CRX_DLLSHARE AcmDatumStd : public AcmSymbolStd
{
  friend class AcmSymbbInternals;
public:
  ACRX_DECLARE_MEMBERS(AcmDatumStd);
  ///<summary>Destructures the datum target standard class.</summary>
  virtual ~AcmDatumStd();
  ///<summary>This class implementation doesn't do anything and returns eNotImplemented if it is not
  ///overloaded in the child class.</summary>
  virtual Acm::ErrorStatus cInit();
  virtual Acm::ErrorStatus getDescription(CString& desc,
                                          CBitmap*& pBmpLoaded,
                                          CBitmap*& pBmpReg,
                                          Adesk::Boolean& enablePref) const;
  virtual void initToEditOriginal();
  virtual void initToEdit();
  ///<summary>Gets the special datum target leader, which expresses to be connected to the hidden target
  ///(the target point is attached to the hidden geometry).</summary>
  AcDbObjectId hiddenLeaderLineId() const;
  ///<summary>Creates a special datum target leader, which expresses to be connected to the hidden target 
  ///(the target point is attached to the hidden geometry).</summary>
  Acm::ErrorStatus setHiddenLeaderLineId(const AcDbObjectId& idType);
  
  Acm::ErrorStatus getHiddenLeaderLtypeName(ACHAR*& pName) const;
  ///<summary>Creates a special datum target leader, which expresses to be connected to the hidden target 
  ///(the target point is attached to the hidden geometry).</summary>
  Acm::ErrorStatus setHiddenLeaderLtypeName(const ACHAR* pName);
  ///<summary>Gets the line type used to draw the borderline of a target symbol.</summary>
  AcDbObjectId borderLineId() const;
  ///<summary>Sets the line type used to draw the borderline of a target symbol.</summary>
  Acm::ErrorStatus setBorderLineId(const AcDbObjectId& idType);
  Acm::ErrorStatus getBorderLtypeName(ACHAR*& pName) const;
  Acm::ErrorStatus setBorderLtypeName(const ACHAR* pName);
  ///<summary>Gets whether to use the leader arrowhead for all leaders. If not, then kNoArrow is used as 
  ///the arrowhead type for this leader.</summary>
  Adesk::Boolean drawLeaderArrow() const;
  ///<summary>Sets whether to use the leader arrowhead for all leaders. If not, then kNoArrow is used as 
  ///the arrowhead type for this leader.</summary>
  void setDrawLeaderArrow(Adesk::Boolean status);
  ///<summary>Gets the hatch angle for hatching inside the rectangular or circular area.</summary>
  double hatchAngle() const;
  ///<summary>Sets the hatch angle for hatching inside the rectangular or circular area.</summary>
  void setHatchAngle(double value);
  ///<summary>Gets the distance between hatch lines.</summary>
  double hatchDist() const;
  ///<summary>Sets the distance between hatch lines.</summary>
  void setHatchDist(double value);
  ///<summary>Gets the special color override for points used in point target and linear target symbols.</summary>
  Adesk::UInt16 pointColor() const;
  ///<summary>Sets the special color override for points used in point target and linear target symbols.</summary>
  void setPointColor(Adesk::UInt16 color);
  const AcCmColor & pointTrueColor() const;
  void setPointTrueColor(const AcCmColor & color);
  ///<summary>Gets the size of the point symbol used in point target and linear target symbols.</summary>
  double pointSize() const;
  ///<summary>Sets the size of the point symbol used in point target and linear target symbols.</summary>
  void setPointSize(double value);
  ///<summary>For internal use only.
  ///Gets the line type used to draw Line temination type</summary>
  AcDbObjectId lineTermLineId() const;
  ///<summary>For internal use only.</summary>
  Adesk::Boolean  useBorderLtypeforLineTerm() const;
  ///<summary>For internal use only.
  /// Sets whether we use Border linetype for Line temination type. If true,
  /// lineTermLineId() will return same as borderLineId(). If sets it false, lineTermLineId()
  /// will return standard leader line type </summary>
  void  setUseBorderLtypeforLineTerm(Adesk::Boolean status);
  ///<summary>Sets the factory default values.</summary>
  virtual Adesk::Boolean setFactoryDefaults();
  
  virtual Acm::ErrorStatus checkVisibility(AcmCTargetBalloon* pData) const;
  virtual Acm::ErrorStatus initFields(AcmCTargetBalloon* pData) const;
  virtual Acm::ErrorStatus setSuggested(AcmCTargetBalloon* pData) const;
  ///<summary>Gets the sub entities of the datum standard.</summary>
  ///<param name="trans">Specifies the transformation matrix for the sub-symbol. The sub-symbol should not
  ///be transformed.</param>
  virtual void getSubEnts(const AcmCSubSymbol* pSymb,
                          AcmCAuxEntArray& subents,
                          const AcGeMatrix3d& trans) const;

  virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
  virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;
  virtual Acad::ErrorStatus dxfInFields(AcDbDxfFiler* filer);
  virtual Acad::ErrorStatus dxfOutFields(AcDbDxfFiler* filer) const;
  ///<summary>Gets the specified service to use for getting sub-entities.</summary>
  AcmCTargetService* getService(Acm::TargetType type) const;

protected:
  ///<summary>Constructs the datum target standard class. The constructor is protected.</summary>
  AcmDatumStd();
  ///<summary>Registers the target service for use.</summary>
  Adesk::Boolean registerService(AcmCTargetService* pNewServ);
  virtual Acm::ErrorStatus createDefaultLTypeBorderLine(const ACHAR* name);
  virtual Acm::ErrorStatus createDefaultLTypeHiddenLeaderLine(const ACHAR* name);
  virtual Acad::ErrorStatus  subGetClassID(CLSID* pClsid) const;

private:
  AcmDatumStdImp* mpImp;
};
DBREF_DECLARE(AcmDatumStd)

///<summary>This class has more complex grip points editing, because it contains more independent sub
///symbols - one balloon and one target sub-symbol. They are closely related to their leaders.</summary>
class ACMSYMBB_CRX_DLLSHARE AcmDatumTarget : public AcmSymbol
{
public:
  ACRX_DECLARE_MEMBERS(AcmDatumTarget);
  ///<summary>This constructor creates the symbol using a specific target type.</summary>
  AcmDatumTarget();
  ///<summary>This constructor creates the symbol using a specific target type.</summary>
  AcmDatumTarget(Acm::TargetType type);
  ///<summary>Destructures the datum target class.</summary>
  virtual ~AcmDatumTarget();
  
  LPCTSTR value(Acm::Variable var) const;
  Acm::ErrorStatus setValue(Acm::Variable var, LPCTSTR value);
  ///<summary>Gets the visibility status of the editing controls. It can be kHidden, kDisabled, kShaded,
  ///kSelectOnly, kVisible, where kDisabled, kShaded and kSelectOnly have the same meaning.</summary>
  Acm::Visibility  variableStatus(Acm::Variable var) const;
  ///<summary>Sets the visibility status of the editing controls.</summary>
  ///<param name="status">It can be kHidden, kDisabled, kShaded, kSelectOnly, kVisible, where kDisabled, 
  ///kShaded and kSelectOnly have the same meaning.</param>
  Acm::ErrorStatus setVariableStatus(Acm::Variable var, Acm::Visibility status);
  
  Acm::ErrorStatus getVarDlgInfo(Acm::Variable var,CVarDlgInfo& v) const;
  Acm::ErrorStatus setVarDlgInfo(Acm::Variable var,const CVarDlgInfo& v);
  Acm::Orientation orientation() const;

  void setOrientation(Acm::Orientation o);
  
  virtual AcmCTarget *getNewTarget(Acm::TargetType type);
  ///<summary>Uses this overloaded member function to set up a symbol properly.</summary>
  virtual Acm::ErrorStatus setSymbolDefaults(AcDbDatabase* pDb = NULL);
  AcmCTargetBalloon* balloon() const;
  AcmCTarget* target() const;
  ///<summary>This function is called during the updating of symbols that takes place when the standard is changed.</summary>
  virtual void regen();
  double balloonRadius() const;
  AcDbObjectId lTypeId() const;
  void setLTypeIdToHidden();
  void setLTypeIdToSolid();
  Adesk::Boolean isHidden() const;

  void setHidden(Adesk::Boolean val);
  ///<summary>Sends a notification if the tip point of the leader has changed.</summary>
  virtual void leaderTipPointChanged(const AcGeVector3d& offset,
                                     const AcmCLeader* leader);
  
  using AcmSymbol::transformBy;
  ///<summary>Transforms all leaders. Default implementation doesn't cover any smart behavior.</summary>
  virtual Acad::ErrorStatus transformBy(const AcGeMatrix3d& xform,
                                        double symbolScale);
  ///<summary>This method mirrors AcmSymbol entity.</summary>
  ///<param name:"xMirrorForm">Transformation matrix with MIRROR element</param>
  ///<param name:"XAxis">Returns mirrored X-Axis of the entity coordinate system</param>
  ///<param name:"YAxis">Returns mirrored Y-Axis of the entity coordinate system</param>
  ///<param name:"normal">Returns mirrored Normal of the entity coordinate system</param>
  virtual void mirrorCoordSys(const AcGeMatrix3d& xMirrorForm,
                              AcGeVector3d& XAxis,
                              AcGeVector3d& YAxis,
                              AcGeVector3d& normal) const;

  virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
  virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;
  virtual Acad::ErrorStatus dxfInFields(AcDbDxfFiler* filer);
  virtual Acad::ErrorStatus dxfOutFields(AcDbDxfFiler* filer) const;
  virtual void updateLeader();
  
  virtual Acm::ErrorStatus attachGeomRef(const void* pGeomRef,
                                         const AcDbObjectId &viewId,
                                         const AcGePoint3d &arrowheadLocation);

  virtual Acm::ErrorStatus attachGeomRef(const void* pGeomRef,
                                         const AcGePoint3d &arrowheadLocation);

  bool setTargetAttach(const AcmDatumTarget* pDatumTarget, const Acm::AttachPosition position);
  virtual Acm::ErrorStatus addLeader(const AcGePoint3dArray& ptrArray);

protected:
  virtual Adesk::Boolean    subWorldDraw(AcGiWorldDraw* pMode);
  ///<summary>Gets the symbol class ID.</summary>
  virtual Acad::ErrorStatus   subGetClassID(CLSID* pClsid) const;
  ///<summary>Sums geometry extents for a leader.</summary
  virtual Acad::ErrorStatus subGetOsnapPoints (AcDb::OsnapMode, INT_PTR,
                                               const AcGePoint3d&,
                                               const AcGePoint3d&,
                                               const AcGeMatrix3d&,
                                               AcGePoint3dArray&,
                                               AcDbIntArray&) const;
  ///<summary>Queries all leaders for their grip points.</summary>
  virtual Acad::ErrorStatus subGetGripPoints(AcGePoint3dArray&,
                                             AcDbIntArray&,
                                             AcDbIntArray&) const;
  ///<summary>Queries all leaders for their grip points.</summary>
  virtual Acad::ErrorStatus   subGetGripPoints(AcDbGripDataPtrArray& grips,
                                               const double curViewUnitSize, 
                                               const int gripSize, 
                                               const AcGeVector3d& curViewDir,
                                               const int bitflags) const;
  ///<summary>Determines the leaders to which indices belong and apply moveGripPointsAt to them.
  ///If the tip point is being edited, then the leaderTipPointChanged() member function of the symbol will be called. 
  ///If the leader contains a landing and it is changed, then the leaderLanding() member function of the symbol is called.</summary>
  virtual Acad::ErrorStatus subMoveGripPointsAt(const AcDbIntArray& indices,
                                                const AcGeVector3d& offset);
  ///<summary>Implemented by calling the getGripPoints member function.</summary>
  virtual Acad::ErrorStatus subGetStretchPoints(AcGePoint3dArray&) const;
  ///<summary>Determines the leaders to which indices belong and apply moveGripPointsAt to them. 
  ///If the tip point is being edited, then the leaderTipPointChanged() member function of the symbol will be called. 
  ///If the leader contains a landing and it is changed, then the leaderLanding() member function of the symbol is called.</summary>
  virtual Acad::ErrorStatus subMoveStretchPointsAt(const AcDbIntArray& indices,
                                                   const AcGeVector3d& offset);
  virtual Acad::ErrorStatus subTransformBy(const AcGeMatrix3d& xform);
  ///<summary>Sums geometry extents for a leader.</summary>
  virtual Acad::ErrorStatus subGetGeomExtents(AcDbExtents&) const;
  ///<summary>Queries the standard for sub-entities for all leaders and then the server for all sub-entities 
  ///for all sub-symbols. All sub-entities are then exploded.</summary>
  virtual Acad::ErrorStatus subExplode(AcDbVoidPtrArray& entitySet) const;
  virtual void              subList() const;

private:
  void updateOrigin();
  AcmCDatumTargetImp* mpImp;
};                      
DBREF_DECLARE(AcmDatumTarget)

///<summary>This symbol defines a common interface for all target types. Extended interfaces are implemented.</summary>
class AcmCTarget : public AcmCSubSymbol
{
public:
  virtual Acm::TargetType type() const = 0;
  virtual double scale() const = 0;

  virtual void getDefaultText(CString& text) const = 0;

  virtual AcGePoint3d recalcLeaderStart(const AcGePoint3d& pnt) const = 0;
  
  virtual Acad::ErrorStatus getGripPoints(AcGePoint3dArray&  gripPoints,
                                          AcDbIntArray&  osnapModes,
                                          AcDbIntArray&  geomIds) const = 0;
  
  virtual Acad::ErrorStatus moveGripPointsAt(const AcDbIntArray& indices,
                                             const AcGeVector3d& offset) = 0;
  ///<summary>Implemented by calling the moveGripPointsAt member function.</summary>
  virtual Acad::ErrorStatus moveStretchPointsAt(const AcDbIntArray& indices,
                                                const AcGeVector3d& offset) = 0;
  ///<summary>Returns the number of grip points used by this class.</summary>
  virtual int numGripPoints() const = 0;

  virtual Adesk::Boolean isKnownGrip(int indice) const = 0;
  virtual Acad::ErrorStatus transformBy(const AcGeMatrix3d& xform) = 0;
};   
///<summary>It is inherited from AcmCTarget. The basic interface is implemented as described in
///AcmCSubSymbol and AcmCTarget.</summary>
class AcmCTargetBalloon : public AcmCTarget
{
public:
  virtual Acm::Orientation orientation() const = 0;

  virtual void setOrientation(Acm::Orientation o) = 0;
};

///<summary>It is inherited from AcmCTarget. The basic interface is implemented as described in 
///AcmCSubSymbol and AcmCTarget.</summary>
class AcmCTargetPoint : public AcmCTarget
{
public:
  virtual Adesk::Boolean drawPoint() const = 0;

  virtual void setDrawPoint(Adesk::Boolean flag) = 0;
};
///<summary>It is inherited from AcmCTarget. The basic interface is implemented as described in 
///AcmCSubSymbol and AcmCTarget.</summary>
class AcmCTargetLine : public AcmCTarget
{
public:
  virtual Adesk::Boolean drawLine() const = 0;

  virtual void setDrawLine(Adesk::Boolean flag) = 0;
  virtual AcGePoint3d endPoint() const = 0;
  virtual void setEndPoint(const AcGePoint3d& point) = 0;
  virtual double factor() const = 0;
  virtual void setFactor(double f) = 0;
  virtual AcGePoint3d midPoint() const = 0;
};

///<summary>It is inherited from AcmCTarget. The basic interface is implemented as described in 
///AcmCSubSymbol and AcmCTarget.</summary>
class AcmCTargetCircularArea : public AcmCTarget
{
public:
  virtual double radius() const = 0;
  virtual void setRadius(double r) = 0;
};
///<summary>It is inherited from AcmCTarget. The basic interface is implemented as described in
///AcmCSubSymbol and AcmCTarget.</summary>
class AcmCTargetRectangularArea : public AcmCTarget
{
public:
  virtual double angle() const = 0;

  virtual void setAngle(double aplha) = 0;
  virtual double xSize() const = 0;
  virtual void setXSize(double x) = 0;
  virtual double ySize() const = 0;
  virtual void setYSize(double y) = 0;
  virtual AcGePoint3d cornerPoint() const = 0;
  virtual void setCornerPoint(const AcGePoint3d& pt) = 0;
  virtual AcGePoint3d anglePoint() const = 0;
  virtual void setAnglePoint(const AcGePoint3d& pt) = 0;
};                      
///<summary>These services do not contain data and they are not filled out to drawing. 
///There are two main entry points to these classes: registerService and getService. They are called
///during constructing of AcmDatumStd. Each service object contains a back pointer to enable it to remove
///its reference if these services are deleted. These services are used internally by AcmDatumStd to draw 
///specific types of datum targets.</summary>
class AcmCTargetService 
{
public:
  ///<summary>Destructures the AcmCTargetService class.</summary>
  virtual ~AcmCTargetService();
  virtual Acm::TargetType serviceType() const = 0;  
  ///<param name="mTransform">Specifies where the graphic representation of the symbol should be transformed.
  ///The sub-entities are created according to this matrix and cannot easily be transformed later.</param>
  virtual void getSubEnts(const AcmCSubSymbol* pData,
                          AcmCAuxEntArray& subEnts,
                          const AcGeMatrix3d& mTransform) const = 0;  

};                      
///<summary>This is class AcmCTargetBalloonService.</summary>
class AcmCTargetBalloonService : public AcmCTargetService 
{
public:
  virtual double radius() const = 0;
  virtual AcGePoint3d defaultSizePosition() const = 0;

protected:
  virtual double gapMultiple() const = 0;
  virtual void   drawOutside(const AcGePoint3d& p1,
                             const AcGePoint3d& p2,
                             AcDbMText& pText,
                             AcmCAuxEntArray& subEnts) const = 0;
};                      

inline 
AcmCTargetService::~AcmCTargetService()
{
}

#endif
